Light = {}
Light.x = 0
Light.y = 0
Light.init_r = 5
Light.end_r = 10
Light.time = 1
Light.ctime = 0
Light.color = 'red'
Light.width = 2
Light.init_hue = 100

function Light:new(x, y, perfil)
    local o = {}
    o.x = x
    o.y = y
    o.init_r = perfil[1]
    o.end_r = perfil[2]
    o.time = perfil[3]
    o.color = perfil[4]
    o.width = perfil[5]
    o.init_hue = perfil[6]
    setmetatable(o, self)
    self.__index = self
    return o
end
function Light:draw()
    local d_r = self.end_r - self.init_r
    local unittime = self.ctime/self.time
    love.graphics.setLineWidth(self.width)
    local c = colors[self.color]
    love.graphics.setColor(c[1], c[2], c[3], self.init_hue*(1 - unittime))
    love.graphics.circle('line', self.x, self.y, self.init_r + d_r*unittime)
end
function Light:update(dt)
    self.ctime = self.ctime + dt
end

Bullet = {}
Bullet.body = nil
Bullet.shape = nil
Bullet.fixture = nil
Bullet.st_x = 0
Bullet.st_y = 0
Bullet.radius = 0
Bullet.last_x = 0
Bullet.last_y = 0
Bullet.angle = 0
Bullet.speed = 1000
Bullet.color = 'white'
Bullet.damage = 10
Bullet.life = 0
Bullet.death = 1
Bullet.owner = 0
Bullet.id = 0

function Bullet:new(x, y, angle, speed, color, damage, radius, death, ownerid, id)
    local o = {}
    o.body = love.physics.newBody(world, x, y, 'dynamic')
    o.shape = love.physics.newCircleShape(radius)
    o.fixture = love.physics.newFixture(o.body, o.shape)
    o.fixture:setUserData({'bullet', id, ownerid, damage})
    o.radius = radius
    o.id = id
    o.st_x = x
    o.st_y = y
    o.angle = angle
    o.speed = speed
    o.color = color
    o.damage = damage
    o.death = death
    o.owner = ownerid
    setmetatable(o, self)
    self.__index = self
    return o
end

function Bullet:update(dt)
    if self.body:isDestroyed() then
        self.life = self.death
    else
        self.body:setLinearVelocity(self.speed*math.cos(self.angle), self.speed*math.sin(self.angle))
        self.life = self.life + dt
        self.last_x = self:getX()
        self.last_y = self:getY()
    end
end

function Bullet:draw()
    if not self.body:isDestroyed() then
        love.graphics.setColor(colors[self.color])
        love.graphics.circle('fill', self:getX(), self:getY(), self:getRadius())
    end
end

function Bullet:getX()
    return self.body:getX()
end
function Bullet:getY()
    return self.body:getY()
end
function Bullet:getRadius()
    return self.shape:getRadius()
end

function Bullet:died()
    if self.life >= self.death then
        return true
    end
    return false
end

Gun = {}

Gun.bullets = nil
Gun.angle = -math.pi/2
Gun.x = 0
Gun.y = 0
Gun.radius = 0
Gun.bullet_radius = 5
Gun.speed = 500
Gun.color = 'white'
Gun.damage = 10
Gun.followcursor = false
Gun.bullet_death = 1
Gun.sound = 'heavyshot'
Gun.light_p = 'l-white'
Gun.ownerid = 0

function Gun:new(o)
    o = o or {}
    o.bullets = {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function Gun:shot()
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle, 
        self.speed, self.color, self.damage, self.bullet_radius, 
        self.bullet_death, self.ownerid, #self.bullets + 1))
    lightEffect(self.x, self.y, self.light_p)
    playSound(self.sound, 0.5)
end

function Gun:update(dt, x, y)
    self.x = x + self.radius*math.cos(self.angle)
    self.y = y + self.radius*math.sin(self.angle)
    if self.followcursor then
        self.angle = math.atan2(love.mouse.getY() - y, love.mouse.getX() - x)
    end
    for i = 1, #self.bullets do
        self.bullets[i]:update(dt)
    end
    for i = 1, #self.bullets do
        if self.bullets[i]:died() then
            if not self.bullets[i].body:isDestroyed() then
                self.bullets[i].body:destroy()
            end
            lightEffect(self.bullets[i].last_x, self.bullets[i].last_y, 'custom',
                {self.bullets[i].radius, self.bullets[i].radius*2, 
                    0.8, self.bullets[i].color, 8, 150})
            table.remove(self.bullets, i)
            break -- I know, it's ugly, but i have only 2 days
        end
    end
end

function Gun:draw()
    love.graphics.setColor(colors[self.color])
    love.graphics.circle('line', self.x, self.y, self.bullet_radius + self.damage/10)
    love.graphics.circle('fill', self.x, self.y, self.bullet_radius + self.damage/10)

    for i = 1, #self.bullets do
        self.bullets[i]:draw()
    end
end
guns = {
--inferior
['WeakGun'] = Gun:new({bullet_radius = 8, speed = 400, 
    color = 'ciano-bright', damage = 5, bullet_death = 4, sound = 'softshot',
    light_p = 'f-ciano'}),
--superior
['SuperFire'] = Gun:new({bullet_radius = 8, speed = 800, bullet_death = 2,
    color = 'orange4', damage = 30, light_p = 'm-orange'}),
['SuperIce'] = Gun:new({bullet_radius = 7, speed = 800, bullet_death = 2,
    color = 'ciano2', damage = 30, light_p = 'm-ciano'}),
--supreme
['UltraEnergy'] = Gun:new({bullet_radius = 9, speed = 800, 
    color = 'yellow3', damage = 60, light_p = 'b-yellow'}),
['UltraNuclear'] = Gun:new({bullet_radius = 15, speed = 1200, 
    color = 'pink2', damage = 60, light_p = 'b-pink2'}),
}
function guns.UltraEnergy:shot()
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle, 
        self.speed, self.color, self.damage, self.bullet_radius))
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle + math.pi/20, 
        self.speed, self.color, self.damage, self.bullet_radius))
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle - math.pi/20, 
        self.speed, self.color, self.damage, self.bullet_radius))
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle + math.pi/10, 
        self.speed, self.color, self.damage, self.bullet_radius))
    table.insert(self.bullets, Bullet:new(self.x, self.y, self.angle - math.pi/10, 
        self.speed, self.color, self.damage, self.bullet_radius))
end
