Enemy = {}
Enemy.body = nil
Enemy.shape = nil
Enemy.fixture = nil
Enemy.speed = 80
Enemy.angle = 0
Enemy.gun = nil
Enemy.shot_freq = 4.8
Enemy.last_shot = 0
Enemy.hp = 30
Enemy.move_len = 0
Enemy.id = 0
Enemy.color1 = 'pink2'
Enemy.color2 = 'silver'


function Enemy:new(ox, oy, radius, id, other)
    local o = {}
    other = other or {}
    o.body = love.physics.newBody(world, ox, oy, 'dynamic')
    o.shape = love.physics.newCircleShape(radius)
    o.fixture = love.physics.newFixture(o.body, o.shape)
    o.fixture:setUserData({'enemy', id, false})
    o.id = id
    o.gun = other.gun or WeakGun:new({radius = 25, x = ox, y = oy})
    o.hp = other.hp or 30
    o.shot_freq = other.shot_freq or 4.8
    o.color1 = other.color1 or 'pink2'
    o.color2 = other.color2 or 'silver'
    o.last_shot = main_clock:getTime()
    setmetatable(o, self)
    self.__index = self
    return o
end

function Enemy:draw()
    love.graphics.setColor(colors[self.color2])
    love.graphics.polygon('fill', self:getX() - self:getRadius(), self:getY(),
        self:getX(), self:getY() - self:getRadius()/2,
        self:getX() + self:getRadius(), self:getY(),
        self:getX(), self:getY() + self:getRadius()/2)
    local c1 = colors[self.color1]
    love.graphics.setColor(c1[1], c1[2], c1[3], 50 + senoid('sin', 120, 1, 0))
    love.graphics.setLineWidth(5)
    love.graphics.circle('line', self:getX(), self:getY(), self:getRadius()/3)
    self.gun:draw()
end
function Enemy:update(dt)
    local userd = self.fixture:getUserData()
    if userd[3] then
        self.hp = self.hp - userd[3]
        self.fixture:setUserData({userd[1], userd[2], false})
    end
    if main_clock:getTime() - self.last_shot > self.shot_freq then
        self:shot()
        self.last_shot = main_clock:getTime()
    end
    self.gun:update(dt, self:getX(), self:getY())
    self.gun.angle = math.atan2(ship:getY() - self:getY(), ship:getX() - self:getX())
    if self.move_len > 0 then
        self.body:setLinearVelocity(self.speed*math.cos(self.angle), 
            self.speed*math.sin(self.angle))
        self.move_len = self.move_len - dt
    else
        self.body:setLinearVelocity(0, 0)
        --gambiarra detected ^^
    end
end

function Enemy:getX()
    return self.body:getX()
end
function Enemy:getY()
    return self.body:getY()
end
function Enemy:getRadius()
    return self.shape:getRadius()
end

function Enemy:shot()
    local noise = love.math.random(0, math.pi/2)
    self.gun:shot()
end
function Enemy:move(angle, len)
    self.move_len = len
    self.angle = angle
end

enemyDict = {
-- {radius, gunName, shot_freq, hp, color1, color2}
{30, 'WeakGun', 1.4, 30, 'pink2', 'silver'},
{50, 'SuperFire', 1.4, 60, 'pink4', 'ciano-bright'},
{40, 'SuperFire', 0.9, 80, 'purple','blue-dark'},
{70, 'UltraNuclear', 2, 160, 'white', 'purple'},
{90, 'UltraEnergy', 0.5, 200, 'black', 'red4'}
}

enemyController = {}
enemyController.list = {}
enemyController.list_freq = {1.9, 2.9, 4.7, 6.7, 8}
enemyController.list_last = {5, 40, 120, 180, 240}
--enemyController.list_last = {0,0,0,0,0}

function enemyController:update(dt)
    for i = 1, #self.list do
        self.list[i]:update(dt)
    end
    for i = 1, #self.list do
        if self.list[i].hp <= 0 then
            playSound('destroy1', 0.3)
            for j = 1, self.list[i]:getRadius()/2 do
                local rx = love.math.random(0, 40)
                local ry = love.math.random(0, 40)
                lightEffect(self.list[i]:getX() + rx - 20, self.list[i]:getY() + ry - 10, 
                    'custom', {self.list[i]:getRadius(), self.list[i]:getRadius()*2,
                    0.3, self.list[i].color2, 3, 200})
            end
            for j = 1, #self.list[i].gun.bullets do
                self.list[i].gun.bullets[j].body:destroy()
            end
            self.list[i].body:destroy()
            table.remove(self.list, i)
            break
        end
    end

    for i = 1, #self.list_freq do
        if main_clock:getTime() - self.list_last[i] > self.list_freq[i] and #self.list < 20 then
            enemyController:create(i)
            self.list_last[i] = main_clock:getTime()
        end
    end

end
function enemyController:draw()
    for i = 1, #self.list do
        self.list[i]:draw()
    end
end

function enemyController:create(id)
    local xpos = love.math.random(100, width-100)

    table.insert(self.list, Enemy:new(xpos,-120, enemyDict[id][1], (#self.list)+1,
        {
        gun = guns[enemyDict[id][2]]:new({radius = enemyDict[id][1]}), 
        shot_freq = enemyDict[id][3],
        hp = enemyDict[id][4],
        color1 = enemyDict[id][5],
        color2 = enemyDict[id][6]
        }))
    self.list[#self.list]:move(math.pi/2, 3)
end


