function love.load()
    width, height = love.window.getDesktopDimensions()
    love.window.setMode(width, height, {vsync = true, fullscreen = true})
    love.mouse.setVisible(false)
    require 'relogio'
    main_clock = relogio:new()
    main_clock:start()
    world = love.physics.newWorld(0, 0)
    world:setCallbacks(beginContact, endContact, preSolve, postSolve)
    require 'utils'
    require 'cursor'
    require 'bullets'
    require 'spaceship'
    require 'enemies'
    ship = Ship:new(width/2, height/2, 30)

    music = love.audio.newSource("Eternal Spaceship War demo.mp3")
    music:setVolume(0.5)
    music:setLooping(true)
    music:play()
    light_effects = {}
    game_over = false
    fade_out = 0
    font1 = love.graphics.newFont('Digitalt.ttf', 120)
    font2 = love.graphics.newFont('vavont.ttf', 50)
    love.graphics.setFont(font1)
    count_for_thanks = 0
    time_for_thanks = 5
    msg1 = "GAME OVER"
    msg2 = "Thank you for playing <3"
    msg1_h = 0
    msg2_h = 0
end

function love.update(dt)
    if not game_over then
        ship:update(dt)
        enemyController:update(dt)
    end
    main_clock:update(dt)
    for i = 1, #light_effects do
        light_effects[i]:update(dt)
    end
    for i = 1, #light_effects do
        if light_effects[i].ctime >= light_effects[i].time then
            table.remove(light_effects, i)
            break
        end
    end
    if game_over then
        if count_for_thanks < time_for_thanks then
            fade_out = fade_out + dt*100
            msg1_h = msg1_h + dt*100
            count_for_thanks = count_for_thanks + dt
        end
        if count_for_thanks >= time_for_thanks then
            msg1_h = msg1_h - dt*510
            msg2_h = msg2_h + dt*100
            if msg2_h >= 255 then
                love.event.quit()
            end
        end
    end
    world:update(dt)
end

function love.draw()
    ship:draw()
    enemyController:draw()
    mouse:draw()
    for i = 1, #light_effects do
        light_effects[i]:draw()
    end
    love.graphics.setColor(0, 0, 0, fade_out)
    love.graphics.rectangle('fill', 0, 0, width, height)
    love.graphics.setColor(255, 80, 80, msg1_h)
    love.graphics.setFont(font1)
    love.graphics.print(msg1, width/2, height/2, 0, 1, 1, 
        font1:getWidth(msg1)/2, font1:getHeight()/2)
    love.graphics.setColor(220, 160, 220, msg2_h)
    love.graphics.setFont(font2)
    love.graphics.print(msg2, width/2, height*0.66, 0, 1, 1, 
        font2:getWidth(msg2)/2, font1:getHeight()/2)

end

function love.mousepressed(mx, my, key)
    if game_over then

    else
        if key == 1 then
            ship:shot(1)
        end
        if key == 2 then
            ship:shot(2)
        end
    end
end

function love.keypressed(key)
    if key == 'escape' then
        love.event.quit()
    end
    if game_over then

    else
        if key == 'w' then
            ship:move('v', -1)
        end
        if key == 's' then
            ship:move('v', 1)
        end
        if key == 'a' then
            ship:move('h', -1)
        end
        if key == 'd' then
            ship:move('h', 1)
        end
        if key == 'k' then
            ship.hp = ship.hp - 5
        end
        if key == 'l' then
            ship.hp = ship.hp + 5
        end
        if key == 'n' then
            ship.mp = ship.mp - 5
        end
        if key == 'm' then
            ship.mp = ship.mp + 5
        end
    end
end

function love.keyreleased(key)
    if game_over then

    else
        if key == 'w' then
            ship:move('v', 1)
        end
        if key == 's' then
            ship:move('v', -1)
        end
        if key == 'a' then
            ship:move('h', 1)
        end
        if key == 'd' then
            ship:move('h', -1)
        end
    end
end

function beginContact(a, b, coll)
    if a:getUserData() and b:getUserData() then
        local a_data = a:getUserData()
        local b_data = b:getUserData()
        if a_data[1] == 'enemy' and b_data[1] == 'bullet' then
            if b_data[3] == 'ship' then
                a:setUserData({a_data[1], a_data[2], b_data[4]})
                b:getBody():destroy()
                playSound('hit', 0.1)
            end
        end
        if a_data[1] == 'bullet' and b_data[1] == 'enemy' then
            if a_data[3] == 'ship' then
                b:setUserData({b_data[1], b_data[2], a_data[4]})
                a:getBody():destroy()
                playSound('hit', 0.1)
            end
        end
        if a_data[1] == 'ship' and b_data[1] == 'bullet' then
            if b_data[3] ~= 'ship' then
                a:setUserData({a_data[1], b_data[4]})
                b:getBody():destroy()
                playSound('hit', 0.3)
            end
        end
        if a_data[1] == 'bullet' and b_data[1] == 'ship' then
            if a_data[3] ~= 'ship' then
                b:setUserData({b_data[1], a_data[4]})
                a:getBody():destroy()
                playSound('hit', 0.3)
            end
        end
        if a_data[1] == 'shield' then
            if b_data[1] == 'bullet' then
                b:getBody():destroy()
            end
        end
        if b_data[1] == 'shield' then
            if a_data[1] == 'bullet' then
                a:getBody():destroy()
            end
        end
    end
end
 
function endContact(a, b, coll)
 
end
 
function preSolve(a, b, coll)
 
end
 
function postSolve(a, b, coll, normalimpulse, tangentimpulse)
 
end