Ship = {}
Ship.body = nil
Ship.shape = nil
Ship.fixture = nil
Ship.speed = 400
Ship.vertical = 0
Ship.horizontal = 0
Ship.r_angle = 0
Ship.b_angle = 0
Ship.gun = nil
Ship.shield = 0
Ship.shield_radius = 40 --add
Ship.shieldb = nil
Ship.shields = nil
Ship.shieldf = nil
Ship.hp = 100
Ship.mp = 100
Ship.id = 'ship'

function Ship:new(x, y, radius)
    local o = o or {}
    o.body = love.physics.newBody(world, x, y, 'dynamic')
    o.shape = love.physics.newCircleShape(radius)
    o.fixture = love.physics.newFixture(o.body, o.shape)
    o.fixture:setUserData({'ship', false})
    o.gun = guns.SuperIce:new({radius = o.shape:getRadius(), followcursor = true, ownerid = 'ship'})
    setmetatable(o, self)
    self.__index = self
    return o
end

function Ship:update(dt)
    local userd = self.fixture:getUserData()
    if userd[2] then
        self.hp = self.hp - userd[2]/2
        self.fixture:setUserData({userd[1], false})
    end
    if self.hp <= 0 then
        game_over = true
        playSound('destroy1', 0.8)
        for i = 1, 5 do
            local rx = love.math.random(0, 60)
            local ry = love.math.random(0, 60)
            lightEffect(self:getX() + rx - 30, self:getY() + ry - 10, 'b-pink2')
        end
        for i = 1, 5 do
            local rx = love.math.random(0, 60)
            local ry = love.math.random(0, 60)
            lightEffect(self:getX() + rx - 30, self:getY() + ry - 10, 'b-blue')
        end
        for i = 1, 9 do
            local rx = love.math.random(0, 60)
            local ry = love.math.random(0, 60)
            lightEffect(self:getX() + rx - 30, self:getY() + ry - 10, 'b-silver')
        end
        for i = 1, 3 do
            local rx = love.math.random(0, 80)
            local ry = love.math.random(0, 80)
            lightEffect(self:getX() + rx - 40, self:getY() + ry - 10, 'red-dramatic')
        end
    end

    self.gun:update(dt, self.body:getX(), self.body:getY())

    self.r_angle = self.r_angle + dt/2
    self.b_angle = self.b_angle - dt/2
    self.body:setLinearVelocity(self.speed*self.horizontal, self.speed*self.vertical)
    if self:getX() < self:getRadius() then
        self:setX(self:getRadius())
    end
    if self:getX() > width - self:getRadius() then
         self:setX(width - self:getRadius())
    end
    if self:getY() < self:getRadius() then
        self:setY(self:getRadius())
    end
    if self:getY() > height - self:getRadius() then
        self:setY(height - self:getRadius())
    end
    
    if self.shield > 0 then --if shield is yet enabled
        self.shield = self.shield - dt/2
        self.shieldb:setX(self:getX())
        self.shieldb:setY(self:getY())
    end
    if self.shield <= 0 then --if time of shield finishes
        self.shield = 0
        if self.shieldb then
            self.shieldb:destroy() --destroy the body of shield
        end
        self.shieldb = nil
        self.shields = nil
        self.shieldf = nil
    end
end
function Ship:activeShield()
    self.shield = 1
    self.shieldb = love.physics.newBody(world, self:getX(), self:getY(), 'dynamic')
    --points of chain shape
    local points = {}
    local aunit = math.pi/6
    for i = 1, 12 do
        table.insert(points, (self:getRadius() + self.shield_radius)*math.cos(aunit*i)) 
        table.insert(points, (self:getRadius() + self.shield_radius)*math.sin(aunit*i))
    end
    self.shields = love.physics.newChainShape(true,points)
    self.shieldf = love.physics.newFixture(self.shieldb, self.shields)
    self.shieldf:setUserData({'shield'})

end

function Ship:draw()
    love.graphics.setBlendMode('add')
    love.graphics.setColor(255, 50, 120, 150*(self.hp/100))
    local l_rad = self:getRadius() + 20
    love.graphics.polygon('fill', self:getX() + l_rad*math.cos(self.r_angle), 
        self:getY() + l_rad*math.sin(self.r_angle),
        self:getX() + l_rad*math.cos(self.r_angle + 2*math.pi/3), 
        self:getY() + l_rad*math.sin(self.r_angle + 2*math.pi/3),
        self:getX() + l_rad*math.cos(self.r_angle - 2*math.pi/3), 
        self:getY() + l_rad*math.sin(self.r_angle - 2*math.pi/3))

    love.graphics.setColor(120, 50, 255, 150*(self.mp/100))
    love.graphics.polygon('fill', self:getX() + l_rad*math.cos(self.b_angle), 
        self:getY() + l_rad*math.sin(self.b_angle),
        self:getX() + l_rad*math.cos(self.b_angle + 2*math.pi/3), 
        self:getY() + l_rad*math.sin(self.b_angle + 2*math.pi/3),
        self:getX() + l_rad*math.cos(self.b_angle - 2*math.pi/3), 
        self:getY() + l_rad*math.sin(self.b_angle - 2*math.pi/3))

    love.graphics.setBlendMode('alpha')
    self.gun:draw()

    love.graphics.setColor(220, 220, 220, 255)
    love.graphics.setLineWidth(1)
    love.graphics.circle('line', self:getX(), self:getY(), self:getRadius())
    love.graphics.circle('fill', self:getX(), self:getY(), self:getRadius())

    love.graphics.setColor(150, 200, 220, 255)
    love.graphics.circle('line', self:getX(), self:getY(), self:getRadius()/2)
    love.graphics.circle('fill', self:getX(), self:getY(), self:getRadius()/2)
    love.graphics.setColor(130, 150, 200, 255)
    love.graphics.circle('line', self:getX(), self:getY(), self:getRadius()/3)
    love.graphics.circle('fill', self:getX(), self:getY(), self:getRadius()/3)

    if self.shield > 0 then
        love.graphics.setLineWidth(self.shield*8)
        love.graphics.setBlendMode('add')
        love.graphics.setColor(255, 0, 0, 180*self.shield)
        love.graphics.circle('line', self:getX(), self:getY(), self:getRadius() + self.shield_radius + senoid('sin', 3, 23, 0))
        love.graphics.setColor(0, 255, 0, 180*self.shield)
        love.graphics.circle('line', self:getX(), self:getY(), self:getRadius() + self.shield_radius + senoid('sin', 3, 24, 1))
        love.graphics.setColor(0, 0, 255, 180*self.shield)
        love.graphics.circle('line', self:getX(), self:getY(), self:getRadius() + self.shield_radius + senoid('sin', 3, 25, 2))
        love.graphics.setBlendMode('alpha')
    end

    --HP
    if self.hp > 50 then
        local sbhp = -(self.hp - 100)
        love.graphics.setColor(sbhp*5, 255, 0, 180)
        love.graphics.rectangle('line', 3, 3, 200, 20, 8)
        love.graphics.setColor(sbhp*5, 255, 0, 20)
        love.graphics.rectangle('fill', 3, 3, 200, 20, 8)
        love.graphics.setColor(sbhp*5, 255, 0, 120)
        love.graphics.rectangle('fill', 3, 3, self.hp*2, 20, 8)
    else
        love.graphics.setColor(255, self.hp*5, 0, 180)
        love.graphics.rectangle('line', 3, 3, 200, 20, 8)
        love.graphics.setColor(255, self.hp*5, 0, 20)
        love.graphics.rectangle('fill', 3, 3, 200, 20, 8)
        love.graphics.setColor(255, self.hp*5, 0, 120)
        love.graphics.rectangle('fill', 3, 3, self.hp*2, 20, 8)
    end
    love.graphics.setColor(220 - self.mp*2, 120, 255, 180)
    love.graphics.rectangle('line', 3, 26, 200, 20, 8)
    love.graphics.setColor(220 - self.mp*2, 120, 255, 20)
    love.graphics.rectangle('fill', 3, 26, 200, 20, 8)
    love.graphics.setColor(220 - self.mp*2, 120, 255, 120)
    love.graphics.rectangle('fill', 3, 26, self.mp*2, 20, 8)

    love.graphics.setLineWidth(1)
    love.graphics.setColor(colors.orange[1], colors.orange[2], colors.orange[3], senoid('sin', 60, 0.6, 20))
    love.graphics.line(self.gun.x, self.gun.y, love.mouse.getX(), love.mouse.getY())
end

function Ship:getX()
    return self.body:getX()
end
function Ship:getY()
    return self.body:getY()
end
function Ship:setX(x)
    self.body:setX(x)
end
function Ship:setY(y)
    self.body:setY(y)
end
function Ship:getRadius()
    return self.shape:getRadius()
end

function Ship:move(dir, q)
    if dir == 'v' then
        self.vertical = self.vertical + q
    end
    if dir == 'h' then
        self.horizontal = self.horizontal + q
    end
end

function Ship:shot(k)
    if k == 1 then
        self.gun:shot(self:getX(), self:getY(), self:getRadius()+1)
    end
    if k == 2 then
        self:activeShield()
    end
end

