function distance(x1, y1, x2, y2)
    return math.sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) )
end
function distanceObj(obj1, obj2)
    return math.sqrt( (obj1.x-obj2.x)*(obj1.x-obj2.x) + (obj1.y-obj2.y)*(obj1.y-obj2.y) )
end

colors = {
    ['white'] = {255, 255, 255},
    ['silver'] = {220, 220, 230},
    ['red'] = {255, 0, 0},
    ['red2'] = {250, 20, 30},
    ['red3'] = {240, 30, 70},
    ['red4'] = {200, 80, 100},
    ['gold'] = {255, 185, 0},
    ['orange'] = {255, 150, 10},
    ['orange2'] = {255, 170, 30},
    ['orange3'] = {255, 90, 10},
    ['orange4'] = {230, 150, 80},
    ['yellow'] = {255, 255, 0},
    ['yellow2'] = {250, 250, 30},
    ['yellow3'] = {210, 210, 150},
    ['green'] = {0, 255, 0},
    ['green2'] = {20, 255, 20},
    ['green3'] = {20, 255, 100},
    ['ciano'] = {10, 255, 255},
    ['ciano-dark'] = {30, 150, 150},
    ['ciano2'] = {20, 170, 255},
    ['ciano-bright'] = {180, 220, 255},
    ['blue'] = {0, 0, 255},
    ['blue2'] = {20, 80, 255},
    ['blue3'] = {50, 30, 255},
    ['blue4'] = {80, 20, 240},
    ['blue-dark'] = {20, 20, 140},
    ['purple'] = {130, 10, 230},
    ['purple2'] = {160, 0, 220},
    ['pink'] = {255, 0, 255},
    ['pink2'] = {200, 0, 255},
    ['pink3'] = {255, 10, 200},
    ['pink4'] = {255, 30, 150},
    ['pink-red'] = {255, 20, 100},
    ['grey'] = {150, 150, 150},
    ['grey-gold'] = {170, 140, 80},
    ['black'] = {0,0,0}
}

sounds = {
    ['heavyshot'] = love.audio.newSource("shot.wav"),
    ['softshot'] = love.audio.newSource('softshot.wav'),
    ['hit'] = love.audio.newSource('hit.wav'),
    ['destroy1'] = love.audio.newSource('destroy1.wav'),
    ['destroy2'] = love.audio.newSource('destroy2.wav')
}

function senoid(func, amp, freq, offset)
    amp = amp/2
    if func == 'sin' then
        return amp*math.sin(freq*main_clock:getTime()) + offset + amp
    elseif func == 'cos' then
        return amp*math.cos(freq*main_clock:getTime()) + offset + amp
    end
end

function playSound(name, volume)
    volume = volume or 1
    local sh = sounds[name]:clone()
    sh:setVolume(volume)
    sh:play()
end

function collisionCheck(obj1, obj2, dt)
    --both objets must have x, y, speed, angle
    --need to know if object is moving
    --enemies can have positive speed and not move
    --i don't want to use physics T.T
    --gambiarra
    local p, q = {obj1.x, obj1.y}, {obj2.x, obj2.y}
    local r = {obj1.x + obj1.speed*math.cos(obj1.angle)*dt,
        obj1.y + obj1.speed*math.sin(obj1.angle)*dt}
    local s = {obj2.x + obj2.speed*math.cos(obj2.angle)*dt,
        obj2.y + obj2.speed*math.sin(obj2.angle)*dt}

    -- t = (q − p) × s / (r × s)
    -- u = (q − p) × r / (r × s)
    local rxs = (r[1]*s[2] - r[2]*s[1])
    local t = (s[1]*(q[2] - p[2]) - s[2]*(q[1] - p[1]))/rxs
    local u = (r[1]*(q[2] - p[2]) - r[2]*(q[1] - p[1]))/rxs

    if rxs ~= 0 and t >= 0 and t <= 1 and u >= 0 and u <= 1 then
        local intr = {p[1] + t*r[1], p[2] + t*r[2]}
        if distance(intr[1], intr[2], obj2.x, obj2.y) < obj2.radius + obj1.radius then
            return true
        end
    else
        return false
    end

end

function collisionCheckPC(obj1, obj2, dt)
    if distanceObj(obj1, obj2) < obj2.radius then
        return true
    end
    local b = {(obj1.x + obj1.speed*math.cos(obj1.angle)*dt - obj1.x),
                (obj1.y + obj1.speed*math.sin(obj1.angle)*dt) - obj1.y}
    local a = {obj1.x - obj2.x, obj1.y - obj2.y}
    local b_mod = distance(0, 0, b[1], b[2])
    local a_1 = (a[1]*b[1] + a[2]*b[2])/b_mod
    local b_unit = {b[1]*b_mod, b[2]*b_mod}
    local a_pr = {b_unit[1]*a_1, b_unit[2]*a_1}
    if distance(0, 0, a_pr[1], a_pr[2]) > b_mod then
        return false
    end
    local D = {obj1.x + a_pr[1], obj1.y + a_pr[2]}
    if distance(D[1], D[2], obj2.x, obj2.y) < obj2.radius then
        return true
    end

end

light_perfis = {
    ['l-white'] = {5, 10, 1, 'white', 2, 120},
    ['l-blue'] = {5, 10, 1, 'blue3', 2, 120},
    ['l-green'] = {5, 10, 1, 'green3', 2, 120},
    ['l-red'] = {5, 10, 1, 'red3', 2, 120},
    ['l-yellow'] = {5, 10, 1, 'yellow', 2, 120},
    ['f-ciano'] = {8, 25, 0.3, 'ciano2', 2, 100},
    ['m-ciano'] = {8, 30, 0.6, 'ciano', 3, 120},
    ['m-pink'] = {8, 30, 0.6, 'pink', 3, 120},
    ['m-orange'] = {8, 30,0.6, 'orange3', 3, 120},
    ['m-pink4'] = {10, 40, 0.6, 'pink4', 3, 120},
    ['b-yellow'] = {10, 60, 0.4, 'yellow', 6, 180},
    ['b-pink2'] = {12, 90, 0.4, 'pink2', 10, 220},
    ['b-blue'] = {12, 110, 0.3, 'blue4', 10, 220},
    ['b-silver'] = {12, 120, 0.2, 'silver', 10, 220},
    ['red-dramatic'] = {30, 200, 0.8, 'red', 20, 255}
}

function lightEffect(x, y, perfil_name, custom)
    if perfil_name == 'custom' then
        table.insert(light_effects, Light:new(x, y, custom))
    else
        table.insert(light_effects, Light:new(x, y, light_perfis[perfil_name]))
    end
end